#qpy:kivy
#
# A simple/portable Judo ScoreBoard.
# Copyright (C) 2016 MiKael NAVARRO
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Judo ScoreBoard.
Copyright (C) 2016 MiKael NAVARRO

A simple/portable Judo ScoreBoard written with Kivy.

"""

# Specific variables for pydoc
__author__ = "MiKael Navarro <klnavarro@gmail.com>"
__date__ = "Wed May 10 2016"
__version__ = "0.1.2"
__credits__ = """Thanks to Sentdex for his tutorials."""

# Include directives
import kivy
kivy.require('1.7.2')
 
from kivy.app import App
from kivy.core.window import Window
from kivy.lang import Builder
from kivy.uix.floatlayout import FloatLayout
from kivy.properties import NumericProperty, ObjectProperty
from kivy.clock import Clock
from kivy.core.audio import SoundLoader


# Global variables
#Window.size = (1280, 720)  # default resolution 16:9
#presentation = Builder.load_file("jsb.kv")

 
#
# Main screen.
#
class JSB(FloatLayout):
    """Main screen."""

    # White scores: I W Y + penalities
    w_ippon = ObjectProperty()   # 1000 - 10pts
    w_wazari = ObjectProperty()  #  100 -  7pts
    w_yuko = ObjectProperty()    #   10 -  5pts
    w_koka = ObjectProperty()    #    1 -  3pts
    w_hansokumake = ObjectProperty()
    w_shido1 = ObjectProperty()
    w_shido2 = ObjectProperty()
    w_shido3 = ObjectProperty()
    w_osaekomi = ObjectProperty()
    # Blue scores: I W Y + penalities
    b_ippon = ObjectProperty()   # 1000 - 10pts
    b_wazari = ObjectProperty()  #  100 -  7pts
    b_yuko = ObjectProperty()    #   10 -  5pts
    b_koka = ObjectProperty()    #    1 -  3pts
    b_hansokumake = ObjectProperty()
    b_shido1 = ObjectProperty()
    b_shido2 = ObjectProperty()
    b_shido3 = ObjectProperty()
    b_osaekomi = ObjectProperty()
    # Chronos
    t_chrono = ObjectProperty()
    o_chrono = ObjectProperty()

    # Golden scores.

    def updt_score(self, btn):
        """Update scores: ippon, wazari, yuko, koka."""

        # White scores
        if btn == "W+I":
            score = self.w_ippon.text and int(self.w_ippon.text) or 0
            self.w_ippon.text = str(score + 1)
            if not self.w_wazari.text: self.w_wazari.text = "0"
            if not self.w_yuko.text: self.w_yuko.text = "0"
            if not self.w_koka.text: self.w_koka.text = "0"
        elif btn == "W-I":
            score = self.w_ippon.text and int(self.w_ippon.text) or 0
            if score > 0: self.w_ippon.text = str(score - 1)

        elif btn == "W+W":
            score = self.w_wazari.text and int(self.w_wazari.text) or 0
            self.w_wazari.text = str(score + 1)
            if not self.w_yuko.text: self.w_yuko.text = "0"
            if not self.w_koka.text: self.w_koka.text = "0"
        elif btn == "W-W":
            score = self.w_wazari.text and int(self.w_wazari.text) or 0
            if score > 0: self.w_wazari.text = str(score - 1)

        elif btn == "W+Y":
            score = self.w_yuko.text and int(self.w_yuko.text) or 0
            self.w_yuko.text = str(score + 1)
            if not self.w_koka.text: self.w_koka.text = "0"
        elif btn == "W-Y":
            score = self.w_yuko.text and int(self.w_yuko.text) or 0
            if score > 0: self.w_yuko.text = str(score - 1)

        elif btn == "W+K":
            score = self.w_koka.text and int(self.w_koka.text) or 0
            self.w_koka.text = str(score + 1)
        elif btn == "W-K":
            score = self.w_koka.text and int(self.w_koka.text) or 0
            if score > 0: self.w_koka.text = str(score - 1)

        # Blue scores
        elif btn == "B+I":
            score = self.b_ippon.text and int(self.b_ippon.text) or 0
            self.b_ippon.text = str(score + 1)
            if not self.b_wazari.text: self.b_wazari.text = "0"
            if not self.b_yuko.text: self.b_yuko.text = "0"
            if not self.b_koka.text: self.b_koka.text = "0"
        elif btn == "B-I":
            score = self.b_ippon.text and int(self.b_ippon.text) or 0
            if score > 0: self.b_ippon.text = str(score - 1)

        elif btn == "B+W":
            score = self.b_wazari.text and int(self.b_wazari.text) or 0
            self.b_wazari.text = str(score + 1)
            if not self.b_yuko.text: self.b_yuko.text = "0"
            if not self.b_koka.text: self.b_koka.text = "0"
        elif btn == "B-W":
            score = self.b_wazari.text and int(self.b_wazari.text) or 0
            if score > 0: self.b_wazari.text = str(score - 1)

        elif btn == "B+Y":
            score = self.b_yuko.text and int(self.b_yuko.text) or 0
            self.b_yuko.text = str(score + 1)
            if not self.b_koka.text: self.b_koka.text = "0"
        elif btn == "B-Y":
            score = self.b_yuko.text and int(self.b_yuko.text) or 0
            if score > 0: self.b_yuko.text = str(score - 1)

        elif btn == "B+K":
            score = self.b_koka.text and int(self.b_koka.text) or 0
            self.b_koka.text = str(score + 1)
        elif btn == "B-K":
            score = self.b_koka.text and int(self.b_koka.text) or 0
            if score > 0: self.b_koka.text = str(score - 1)

        else:  # unknown
            print("Btn " + btn + " pressed")

    def updt_white_score(self, btn):
        self.updt_score("W"+btn)

    def updt_blue_score(self, btn):
        self.updt_score("B"+btn)

    # Penalties.

    def updt_penalties(self, btn):
        """Update penalties: hansokumake, shido 1-3."""

        # White penalties
        if btn == "W+H":
            self.w_hansokumake.text = "*"
        elif btn == "W-H":
            self.w_hansokumake.text = ""

        elif btn == "W+S1":
            self.w_shido1.text = "*"
        elif btn == "W-S1":
            self.w_shido1.text = ""

        elif btn == "W+S2":
            self.w_shido2.text = "*"
        elif btn == "W-S2":
            self.w_shido2.text = ""

        elif btn == "W+S3":
            self.w_shido3.text = "*"
        elif btn == "W-S3":
            self.w_shido3.text = ""

        # Blue penalties
        elif btn == "B+H":
            self.b_hansokumake.text = "*"
        elif btn == "B-H":
            self.b_hansokumake.text = ""

        elif btn == "B+S1":
            self.b_shido1.text = "*"
        elif btn == "B-S1":
            self.b_shido1.text = ""

        elif btn == "B+S2":
            self.b_shido2.text = "*"
        elif btn == "B-S2":
            self.b_shido2.text = ""

        elif btn == "B+S3":
            self.b_shido3.text = "*"
        elif btn == "B-S3":
            self.b_shido3.text = ""

        else:  # unknown
            print("Btn " + btn + " pressed")

    def updt_white_penalties(self, btn):
        self.updt_penalties("W"+btn)

    def updt_blue_penalties(self, btn):
        self.updt_penalties("B"+btn)

    # Chronometers

    def countdown_white_osaekomi(self, state):
        if state == "down":
            self.w_osaekomi.text = "<"
            self.b_osaekomi.text = ""  # reset blue osaekomi
            self.b_osaekomi.state = "normal"
            self.o_chrono.state = "down"  # start osaekomi chrono
        else:
            self.w_osaekomi.text = ""
            self.o_chrono.state = "normal"  # reset osaekomi chrono
            self.o_chrono.text = "20"

    def countdown_blue_osaekomi(self, state):
        if state == "down":
            self.b_osaekomi.text = ">"
            self.w_osaekomi.text = ""  # reset white osaekomi
            self.w_osaekomi.state = "normal"
            self.o_chrono.state = "down"  # start osaekomi chrono
        else:
            self.b_osaekomi.text = ""
            self.o_chrono.state = "normal"  # reset osaekomi chrono
            self.o_chrono.text = "20"

    def osaekomi_chrono(self, state):
        if state == "down":
            if self.w_osaekomi.state == "down" or self.b_osaekomi.state == "down":
                # ok, restart osaekomi chrono  
                self.t_chrono.state = "down"  # and restart general chrono too
            else:
                # revert, white/blue not selected!
                self.o_chrono.state = "normal"
        else:
            # pause osaekomi chrono
            self.t_chrono.state = "normal"  # and pause general chrono too

    def update(self, dt):
        """Scheduled every 1 second to update chronometers."""

        #print(dt)

        if self.t_chrono.state == "down":
            minutes, seconds = self.t_chrono.text.split(":")
            chrono = int(minutes) * 60 + int(seconds) - 1
            if chrono >= 0:
                self.t_chrono.text = "%d:%02d" % (chrono / 60, chrono % 60)
            else:
                self.t_chrono.state = "normal"
                beep.play()  # beep!

        if self.o_chrono.state == "down": 
            seconds = int(self.o_chrono.text) - 1
            if seconds >= 0:
                self.o_chrono.text = "%02d" % (seconds,) 
            else:
                self.o_chrono.state = "normal"
                beep.play()  # beep!
                if self.w_osaekomi.text != "": self.updt_white_score("+I")
                if self.b_osaekomi.text != "": self.updt_blue_score("+I")
                self.t_chrono.state = "normal"


#
# Main App class.
#
class JSBApp(App):
    """Judo ScoreBoard."""

    def on_pause(self):
        # Here we can save data if needed
        return True

    def on_resume(self):
        # Here we can check if any data needs replacing (usually nothing)
        pass

    def build(self):
        # Initializes the application; it will be called only once.
        jsb = JSB()
        Clock.schedule_interval(jsb.update, 1)
        return jsb


#
# Main entry point:
#
if __name__ == '__main__':

    # Prepare sound
    beep = SoundLoader.load('snd/beep.ogg')
    bell = SoundLoader.load('snd/bell.mp3')
    buzzer = SoundLoader.load('snd/buzzer.mp3')
    if __debug__ and beep:
        print("Sound found at %s" % beep.source)
        print("Sound is %.3f seconds" % beep.length)
        #beep.play()

    JSBApp().run()
